@@ -17,23 +17,51 @@ alg_map() ->
 		#{ <<"alg">> => <<"A192KW">> },
 		#{ <<"alg">> => <<"A256KW">> },
 		#{ <<"alg">> => <<"dir">> },
+		#{ <<"alg">> => <<"ECDH-1PU">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A128GCMKW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A192GCMKW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A256GCMKW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A128KW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A192KW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+A256KW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+C20PKW">> },
+		#{ <<"alg">> => <<"ECDH-1PU+XC20PKW">> },
 		#{ <<"alg">> => <<"ECDH-ES">> },
+		#{ <<"alg">> => <<"ECDH-ES+A128GCMKW">> },
+		#{ <<"alg">> => <<"ECDH-ES+A192GCMKW">> },
+		#{ <<"alg">> => <<"ECDH-ES+A256GCMKW">> },
 		#{ <<"alg">> => <<"ECDH-ES+A128KW">> },
 		#{ <<"alg">> => <<"ECDH-ES+A192KW">> },
 		#{ <<"alg">> => <<"ECDH-ES+A256KW">> },
-		#{ <<"alg">> => <<"ECDH-ES+A256KW">> },
+		#{ <<"alg">> => <<"ECDH-ES+C20PKW">> },
+		#{ <<"alg">> => <<"ECDH-ES+XC20PKW">> },
 		#{ <<"alg">> => <<"A128GCMKW">> },
 		#{ <<"alg">> => <<"A192GCMKW">> },
 		#{ <<"alg">> => <<"A256GCMKW">> },
 		?LET({P2C, P2S},
 			{integer(1, 256), binary()},
-			#{ <<"alg">> => <<"PBES2-HS256+A128KW">>, <<"p2c">> => P2C, <<"p2s">> => base64url:encode(P2S) }),
+			#{ <<"alg">> => <<"PBES2-HS256+A128GCMKW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
 		?LET({P2C, P2S},
 			{integer(1, 256), binary()},
-			#{ <<"alg">> => <<"PBES2-HS384+A192KW">>, <<"p2c">> => P2C, <<"p2s">> => base64url:encode(P2S) }),
+			#{ <<"alg">> => <<"PBES2-HS384+A192GCMKW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
 		?LET({P2C, P2S},
 			{integer(1, 256), binary()},
-			#{ <<"alg">> => <<"PBES2-HS512+A256KW">>, <<"p2c">> => P2C, <<"p2s">> => base64url:encode(P2S) })
+			#{ <<"alg">> => <<"PBES2-HS512+A256GCMKW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
+		?LET({P2C, P2S},
+			{integer(1, 256), binary()},
+			#{ <<"alg">> => <<"PBES2-HS256+A128KW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
+		?LET({P2C, P2S},
+			{integer(1, 256), binary()},
+			#{ <<"alg">> => <<"PBES2-HS384+A192KW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
+		?LET({P2C, P2S},
+			{integer(1, 256), binary()},
+			#{ <<"alg">> => <<"PBES2-HS512+A256KW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
+		?LET({P2C, P2S},
+			{integer(1, 256), binary()},
+			#{ <<"alg">> => <<"PBES2-HS512+C20PKW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) }),
+		?LET({P2C, P2S},
+			{integer(1, 256), binary()},
+			#{ <<"alg">> => <<"PBES2-HS512+XC20PKW">>, <<"p2c">> => P2C, <<"p2s">> => jose_jwa_base64url:encode(P2S) })
 	]).
 
 enc_map() ->
@@ -43,7 +71,9 @@ enc_map() ->
 		#{ <<"enc">> => <<"A256CBC-HS512">> },
 		#{ <<"enc">> => <<"A128GCM">> },
 		#{ <<"enc">> => <<"A192GCM">> },
-		#{ <<"enc">> => <<"A256GCM">> }
+		#{ <<"enc">> => <<"A256GCM">> },
+		#{ <<"enc">> => <<"C20P">> },
+		#{ <<"enc">> => <<"XC20P">> }
 	]).
 
 jwk_encryptor_gen() ->
@@ -68,31 +98,52 @@ jwk_encryptor_gen() ->
 						end);
 				{<<"A128KW">>, _} ->
 					K = crypto:strong_rand_bytes(16),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"A192KW">>, _} ->
 					K = crypto:strong_rand_bytes(24),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"A256KW">>, _} ->
 					K = crypto:strong_rand_bytes(32),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A128CBC-HS256">>} ->
 					K = crypto:strong_rand_bytes(32),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A192CBC-HS384">>} ->
 					K = crypto:strong_rand_bytes(48),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A256CBC-HS512">>} ->
 					K = crypto:strong_rand_bytes(64),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A128GCM">>} ->
 					K = crypto:strong_rand_bytes(16),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A192GCM">>} ->
 					K = crypto:strong_rand_bytes(24),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"dir">>, <<"A256GCM">>} ->
 					K = crypto:strong_rand_bytes(32),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
+				{<<"dir">>, <<"C20P">>} ->
+					K = crypto:strong_rand_bytes(32),
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
+				{<<"dir">>, <<"XC20P">>} ->
+					K = crypto:strong_rand_bytes(32),
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
+				{<<"ECDH-1PU", _/binary>>, _} ->
+					?LET(CurveId,
+						ec_curve(),
+						begin
+							VStaticKeypair = {VStaticSecret, VStaticPublic} = ec_keypair(CurveId),
+							UStaticKeypair = {UStaticSecret, UStaticPublic} = ec_keypair(CurveId),
+							UEphemeralKeypair = {UEphemeralSecret, UEphemeralPublic} = ec_keypair(CurveId),
+							VStaticSecretKey = jose_jwk:from_key(VStaticSecret),
+							VStaticPublicKey = jose_jwk:from_key(VStaticPublic),
+							UStaticSecretKey = jose_jwk:from_key(UStaticSecret),
+							UStaticPublicKey = jose_jwk:from_key(UStaticPublic),
+							UEphemeralSecretKey = jose_jwk:from_key(UEphemeralSecret),
+							UEphemeralPublicKey = jose_jwk:from_key(UEphemeralPublic),
+							{{ecdh_1pu, VStaticKeypair, UStaticKeypair, UEphemeralKeypair}, JWE, {{VStaticSecretKey, VStaticPublicKey}, {UStaticSecretKey, UStaticPublicKey}, {UEphemeralSecretKey, UEphemeralPublicKey}}}
+						end);
 				{<<"ECDH-ES", _/binary>>, _} ->
 					?LET(CurveId,
 						ec_curve(),
@@ -103,23 +154,29 @@ jwk_encryptor_gen() ->
 							AlicePublicJWK = jose_jwk:from_key(AlicePublicKey),
 							BobPrivateJWK = jose_jwk:from_key(BobPrivateKey),
 							BobPublicJWK = jose_jwk:from_key(BobPublicKey),
-							{{ecdh, AliceKeypair, BobKeypair}, JWE, {{AlicePrivateJWK, AlicePublicJWK}, {BobPrivateJWK, BobPublicJWK}}}
+							{{ecdh_es, AliceKeypair, BobKeypair}, JWE, {{AlicePrivateJWK, AlicePublicJWK}, {BobPrivateJWK, BobPublicJWK}}}
 						end);
 				{<<"A128GCMKW">>, _} ->
 					K = crypto:strong_rand_bytes(16),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"A192GCMKW">>, _} ->
 					K = crypto:strong_rand_bytes(24),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"A256GCMKW">>, _} ->
 					K = crypto:strong_rand_bytes(32),
-					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(K) })};
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
+				{<<"C20PKW">>, _} ->
+					K = crypto:strong_rand_bytes(32),
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
+				{<<"XC20PKW">>, _} ->
+					K = crypto:strong_rand_bytes(32),
+					{K, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(K) })};
 				{<<"PBES2", _/binary>>, _} ->
 					?LET(Key,
 						binary(),
 						begin
-							Password = base64url:encode(Key),
-							{Password, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => base64url:encode(Password) })}
+							Password = jose_jwa_base64url:encode(Key),
+							{Password, JWE, jose_jwk:from_map(#{ <<"kty">> => <<"oct">>, <<"k">> => jose_jwa_base64url:encode(Password) })}
 						end)
 			end
 		end).
@@ -128,9 +185,21 @@ ec_curve() ->
 	oneof([
 		secp256r1,
 		secp384r1,
-		secp521r1
+		secp521r1,
+		x25519,
+		x448
 	]).
 
+ec_keypair(x25519) ->
+	SecretJWK = jose_jwk:generate_key({okp, 'X25519'}),
+	{_, SecretKey} = jose_jwk:to_key(SecretJWK),
+	{_, PublicKey} = jose_jwk:to_public_key(SecretJWK),
+	{SecretKey, PublicKey};
+ec_keypair(x448) ->
+	SecretJWK = jose_jwk:generate_key({okp, 'X448'}),
+	{_, SecretKey} = jose_jwk:to_key(SecretJWK),
+	{_, PublicKey} = jose_jwk:to_public_key(SecretJWK),
+	{SecretKey, PublicKey};
 ec_keypair(CurveId) ->
 	ECPrivateKey = #'ECPrivateKey'{parameters=ECParameters, publicKey=Octets0} = public_key:generate_key({namedCurve, pubkey_cert_records:namedCurves(CurveId)}),
 	Octets = case Octets0 of
@@ -143,7 +212,7 @@ ec_keypair(CurveId) ->
 	ECPublicKey = {ECPoint, ECParameters},
 	{ECPrivateKey, ECPublicKey}.
 
-modulus_size()  -> integer(1024, 1280). % integer(256, 8192) | pos_integer().
+modulus_size()  -> integer(1048, 1280). % integer(256, 8192) | pos_integer().
 exponent_size() -> return(65537).   % pos_integer().
 
 rsa_keypair(ModulusSize) ->
@@ -163,11 +232,18 @@ prop_encrypt_and_decrypt() ->
 			{Keys, JWE, JWKs, binary()}),
 		begin
 			case {Keys, JWKs} of
-				{{ecdh, _, _}, {{AlicePrivateJWK, _AlicePublicJWK}, {BobPrivateJWK, BobPublicJWK}}} ->
+				{{ecdh_1pu, _, _, _}, {{VStaticSecretKey, VStaticPublicKey}, {UStaticSecretKey, UStaticPublicKey}, {UEphemeralSecretKey, UEphemeralPublicKey}}} ->
 					JWEMap = jose_jwe:to_map(JWE),
-					Encrypted = jose_jwk:box_encrypt(PlainText, JWEMap, BobPublicJWK, AlicePrivateJWK),
+					Encrypted = jose_jwk:box_encrypt_ecdh_1pu(PlainText, JWEMap, VStaticPublicKey, UStaticSecretKey, UEphemeralSecretKey),
 					CompactEncrypted = jose_jwe:compact(Encrypted),
-					Decrypted = {_, NewJWE} = jose_jwk:box_decrypt(Encrypted, BobPrivateJWK),
+					Decrypted = {_, NewJWE} = jose_jwk:box_decrypt_ecdh_1pu(Encrypted, UStaticPublicKey, VStaticSecretKey),
+					{PlainText, NewJWE} =:= Decrypted
+					andalso {PlainText, NewJWE} =:= jose_jwe:block_decrypt({UStaticPublicKey, VStaticSecretKey, UEphemeralPublicKey}, CompactEncrypted);
+				{{ecdh_es, _, _}, {{AlicePrivateJWK, _AlicePublicJWK}, {BobPrivateJWK, BobPublicJWK}}} ->
+					JWEMap = jose_jwe:to_map(JWE),
+					Encrypted = jose_jwk:box_encrypt_ecdh_es(PlainText, JWEMap, BobPublicJWK, AlicePrivateJWK),
+					CompactEncrypted = jose_jwe:compact(Encrypted),
+					Decrypted = {_, NewJWE} = jose_jwk:box_decrypt_ecdh_es(Encrypted, BobPrivateJWK),
 					{PlainText, NewJWE} =:= Decrypted
 					andalso {PlainText, NewJWE} =:= jose_jwk:block_decrypt(CompactEncrypted, BobPrivateJWK);
 				{{rsa, _, _}, {PrivateJWK, PublicJWK}} ->
